package org.isakiev.fileManager.contentModel;

import org.isakiev.fileManager.FileManagerException;
import org.isakiev.fileManager.contentManagers.IContentManager;
import org.isakiev.fileManager.contentManagers.IProcessInfoListener;
import org.isakiev.fileManager.entities.*;

import java.util.List;
import java.io.File;
import java.util.ArrayList;
import java.util.logging.*;

/**
 * This class represents model containing the tree of a file system. 
 * @author Isakiev Ruslan
 */
public class ContentModel implements IContentModel {
	
	private IDirEntity myRoot;
	private IContentManager myContentManager;
	private ArrayList<IContentModelListener> myListeners;
	private Logger myLogger;
	
	/**
	 * Main constructor.
	 * @param contentManager The content manager, which is used to get
	 * information about unexplored directories.
	 */
	public ContentModel(IContentManager contentManager) {
		myLogger = Logger.getLogger("org.isakiev.fileManager");
		if (contentManager == null) {
			myLogger.severe("Content model can't work with 'null' ContentManager!");
			throw new FileManagerException("Content model can't work with 'null' ContentManager!");
		}
		myContentManager = contentManager;
		myRoot = myContentManager.getRoot();
		myListeners = new ArrayList<IContentModelListener>();
		
	}

	
	/**
	 * Returns the root of the file system tree.
	 * @return The tree root (its children are the root folders of file system). 
	 */
	public IDirEntity getRoot() {
		return myRoot;
	}
	
	/**
	 * Checks if DirEntity has to explored again. 
	 * (If one of the child's date, size or name was changed.
	 * @param dir directory to check
	 * @return result of the check
	 */
	private boolean haveToExplore(IDirEntity dir) {
		boolean toExplore = false;
		if (dir.isTreeRoot() || dir.isArchiveEntity() || dir.isCompressedEntity()) {
			toExplore = false;
		} else {
			List<IEntity> children = dir.getChildren();
			//check if number of files in directory changed
			File[] files = dir.getFile().listFiles();
			int filesNumber = (files == null ? 0 : files.length);
			if (filesNumber != children.size()) {
				toExplore = true;
			} else {          
				//check if name,size or last modified date were changed
				for (IEntity entity:children) {
					File f = entity.getFile();
					if (entity.getLastModified() != f.lastModified() ||
						entity.getSize() != f.length() ||
						entity.getShortName() != f.getName() ) {
						
						toExplore = true;
						break;
					}
				}
				
			}
		} 
		if (toExplore) {
			//fireModelDataChanged();
		}
		return toExplore;
	}
	
	/**
	 * Returns children of the specified parent.
	 * @param parent parent.
	 * @return children.
	 */
	public List<IEntity> getChildren(IDirEntity parent) {
		//System.out.println("Expl: "+parent.getFullName());
		boolean toExplore;
		if (parent.isExplored()) {
			toExplore = haveToExplore(parent);
		} else {
			toExplore = true;
		}
		if (toExplore) {
			myContentManager.exploreDir(parent);
		}
		return parent.getChildren();
	}
	
	/**
	 * Returns entity by entity path and creates all neccessary entities 
	 * on the way from the root (only if these entities are actually existing).
	 * Default path separator is used.
	 * @param path path to the entity
	 * @return entity or root directory if path does not exist
	 */
	public IEntity getEntity(String path) {
		return getEntity(path, File.separator);
	}
	
	/**
	 * Returns entity by entity path and creates all neccessary entities 
	 * on the way from the root (only if these entities are actually existing).
	 * @param path path to the entity
	 * @param pathSeparator path separator
	 * @return entity or root directory if path does not exist
	 */
	public IEntity getEntity(String path, String pathSeparator) {

		if (path == null) {
			return null;
		}
		String curPath = path;
		ArrayList<String> list = new ArrayList<String>();
		
		//Splitting path name
		int j=0;
		do {
			int index = curPath.indexOf(pathSeparator);
			String value;
			if (index == 0) {
				value = "";
				curPath = curPath.substring(1);
			} else if (index > 0) {
				value = curPath.substring(0,index);
				curPath = curPath.substring(index+1);
			} else {
				list.add(j,curPath);
				break;
			}
			if (j == 0) {
				value = value + pathSeparator;
			}
			list.add(j,value);
			j++;
		} while (curPath.length() > 0);
		
		if (myLogger.isLoggable(Level.FINE)) {
			myLogger.fine("getting entity: path=\""+path+"\" separator=\""+pathSeparator+"\"");
		}
		
		IDirEntity dir = getRoot();
		List<IEntity> rootChildren = getChildren(getRoot());
		IEntity result = (rootChildren.size() == 1 ? getChildren(getRoot()).get(0) : getChildren(getRoot()).get(1));
		boolean found = false;
		
		for (int i = 0; i < list.size(); i++) {
			String current = list.get(i);
			if (myLogger.isLoggable(Level.FINER)) {
				myLogger.finer("path component "+i+": "+current);
			}

			List<IEntity> children = getChildren(dir);
			if (children.size() == 0) {
				if (myLogger.isLoggable(Level.FINER)) {
					myLogger.finer("empty list of children!");
				}
				result = dir;
				break;
			}

			IEntity entity = null;
			for (IEntity e:children) {
				if (e.getShortName().equals(current)) {
					entity = e;
					break;
				}
			}
			//if not found
			if (entity == null) {
				if (myLogger.isLoggable(Level.FINER)) {
					myLogger.finer("entity with equal name wasn't found!");
				}
				break;
			}
			//if found
			if (i == list.size() - 1) {
				result = entity;
				found = true;
				break;
			} else {
				if (entity.hasChildren()) {
					dir = (IDirEntity) entity;
				} else {
					if (myLogger.isLoggable(Level.FINEST)) {
						myLogger.finest("search process isn't finished but current entity has no children!");
					}
					result = dir;
					break;
				}
			}
		}
		if (myLogger.isLoggable(Level.FINE)) {
			if (found) {
				myLogger.fine("method found an entity specified by path string.");
			} else {
				myLogger.fine("entity wasn't found! Returned entity = \""+result.getFullName()+"\".");
			}
		}
		return result;
	}
	
	
	/**
	 * Adds model listener.
	 * @param listener listener
	 */
	public void addModelListener(IContentModelListener listener) {
		myListeners.add(listener);
	}
	/**
	 * Removes model listener.
	 * @param listener listener
	 */
	public void removeModelListener(IContentModelListener listener) {
		myListeners.remove(listener);
	}
	
	private void fireModelDataChanged() {
		for (IContentModelListener listener:myListeners) {
			listener.contentModelDataChanged();
		}
	}
	
	/**
	 * Renames / moves entity to the specified directory
	 * @param entity entity to rename / to move
	 * @param newName new name for this entity
	 * @param listener listens to the process events
	 * @return true if operation completed successfully
	 */
	public boolean renMove(IEntity entity, String newName, IProcessInfoListener listener){
		boolean result = myContentManager.renMove(entity,newName,listener);
		myContentManager.exploreDir((IDirEntity) (entity.getParent()));
		IEntity newEntity = getEntity(newName);
		if (!newEntity.isDiskEntity()) {
			myContentManager.exploreDir((IDirEntity) (newEntity.getParent()));
		}
		fireModelDataChanged();
		return result;
	}
	
	/**
	 * Copies entity. 
	 * @param entity entity to copy
	 * @param substring substrins in the source file names which needs to te 
	 * replaced to get file name for copied file.
	 * @param replacement replacement for the substring
	 * @param listener listens to the process events
	 */
	public void copy(IEntity entity, String substring, String replacement, IProcessInfoListener listener){
		myContentManager.copy(entity, substring, replacement, listener);
		IEntity newEntity = getEntity(entity.getFile().getAbsolutePath().replace(substring, replacement));
		//if (!newEntity.isDiskEntity()) {
			myContentManager.exploreDir((IDirEntity) (newEntity.getParent()));
		//}
		fireModelDataChanged();
	}
	
	/**
	 * Deletes the specified entity
	 * @param entity entity to delete
	 * @param listener listens to the process events
	 */
	public void delete(IEntity entity, IProcessInfoListener listener){
		myContentManager.delete(entity,listener);
		myContentManager.exploreDir((IDirEntity) (entity.getParent()));
		fireModelDataChanged();
	}
	
	/**
	 * Creates subdirectory with the specified name in the parent directory. 
	 * @param parentDirectory parent directory
	 * @param name new directory's name
	 * @return true if operation completed successfully
	 */
	public boolean createDirectory(IDirEntity parentDirectory, String name, IProcessInfoListener processInfoListener) {
		boolean result = myContentManager.createDirectory(parentDirectory, name, processInfoListener);
		myContentManager.exploreDir(parentDirectory);
		fireModelDataChanged();
		return result;
	}
	
	/**
	 * Checks if entity exists.
	 * @param entity entity to check
	 * @return result of the check
	 */
	public boolean entityExists(IEntity entity) {
		boolean exists;
		if (entity.isTreeRoot() || entity.isDiskEntity()) {
			exists = true;
		} else {
			exists = entity.getFile().exists();
		}
		/*if (!exists) {
			fireModelDataChanged();
		}*/
		return exists;
	}
	
	/**
	 * Gets the nearest existing parent. 
	 * @param unexistingEntity unexisting entity
	 * @return the nearest existing parent
	 */
	public IEntity getEntityInsteadOfUnexisting(IEntity unexistingEntity) {
		IEntity entity = unexistingEntity;
		while (!entityExists(entity) && !entity.isDiskEntity()) {
			entity = entity.getParent();
		}
		return entity;
	}


}
