package org.isakiev.fileManager.contentViewers.nc;

import org.isakiev.fileManager.entities.IDirEntity;
import org.isakiev.fileManager.entities.IEntity;

/**
 * Interface for the panel model.
 * @author Ruslan Isakiev
 */
interface INCPanelModel {
	
	/**
	 * Gets number of columns in the table
	 * @return number of columns
	 */
	public int getColumnsNumber();
	
	/**
	 * Gets number of rows in the table
	 * @return number of rows
	 */
	public int getRowsNumber();
	
	/**
	 * Changes table structure according to new number of rows.
	 * @param newRowsNumber new number of rows
	 */
	public void setRowsNumber(int newRowsNumber);
	
	/**
	 * Returns number of files in current directory.
	 * @return number of files
	 */
	public int getNumberOfFilesInCurrentDirectory();
	
	/**
	 * Returns total size of the files in current directory.
	 * @return total size
	 */
	public long getSizeOfFilesInCurrentDirectory();
	
	/**
	 * Returns current directory entity
	 * @return current directory entity
	 */
	public IEntity getCurrentDirectoryEntity();
	
	/**
	 * Returns selected entity or <code>null</code> if nothing is selected
	 * @return selected entity or <code>null</code> if nothing is selected
	 */
	public IEntity getSelectedEntity();
	
	/**
	 * Checks if cell with <code>row</code> and <code>column</code> 
	 * coordinates is selected.
	 * @param row row 
	 * @param column column
	 * @return result of the check
	 */
	public boolean isSelected(int row, int column);
	
	/**
	 * Checks if cell with <code>row</code> and <code>column</code> 
	 * coordinates is empty.
	 * @param row row 
	 * @param column column
	 * @return result of the check
	 */
	public boolean isEmpty(int row, int column);
	
	/**
	 * Checks if cell with <code>row</code> and <code>column</code> 
	 * coordinates is an upper level marker.
	 * @param row row 
	 * @param column column
	 * @return result of the check
	 */
	public boolean isUpperLevelMarker(int row, int column);
	
	/**
	 * Gets element in the specified position.
	 * if position is empty.
	 * @param row row
	 * @param column column
	 * @return element in the specified position or <code>null</code> 
	 * if position is empty.
	 */
	public IEntity getElementAt(int row, int column);
	
	/**
	 * Checks if panel model can process enter action itself.
	 * For example move to the upper directory or to the subdirectory.
	 * @return result of the check.
	 */
	public boolean canProcessEnterAction();
	
	/**
	 * Processes enter actions if it is possible.
	 */
	public void processEnterAction();
	
	/**
	 * Sets current element. 
	 * @param directory new entity which will be set as current directory.
	 * @param selectedElement element to select
	 */
	public void setCurrentElement(IDirEntity directory, IEntity selectedElement);
	

	/**
	 * Moves selection to the specified direction if possible.
	 * @param direction KeyEvent.VK_UP, KeyEvent.VK_DOWN, KeyEvent.VK_LEFT or KeyEvent.VK_RIGHT
	 */
	public void moveSelection(int direction);
	
	/**
	 * Moves selection to the specified position.
	 * @param row row to select
	 * @param column column to select
	 */
	public void moveSelection(int row, int column);
	
	/**
	 * Sets file system tree root as a current directory.
	 */
	public void setRootAsCurrentDirectory();
	
	/**
	 * Adds listener.
	 * @param listener listener
	 */
	public void addPanelModelListener(INCPanelModelListener listener);
	
	/**
	 * Removes listener
	 * @param listener listener
	 */
	public void removePanelModelListener(INCPanelModelListener listener);
}
