package org.isakiev.fileManager.contentViewers.nc;

import java.awt.event.KeyEvent;
import java.util.*;
import java.util.logging.*;

import org.isakiev.fileManager.FileManagerException;
import org.isakiev.fileManager.contentModel.IContentModel;
import org.isakiev.fileManager.entities.*;

class NCPanelModel implements INCPanelModel {
	
	private IContentModel myModel;
	private IDirEntity myTreeRoot;
	private List<IEntity> myRootDirs;
	private IDirEntity myCurrentDir;
	private List<IEntity> myCurrentChildren;
	private boolean myCanGoToUpperDir; //X
	
	private int myRowsNumber;
	private int myColumnsNumber;
	
	private int myShift;
	private int mySelectedRow;
	private int mySelectedColumn;
	
	private int myNumberOfFilesInCurrentDirectory;
	private long mySizeOfFilesInCurrentDirectory;
	
	private Logger myLogger;
	
	private ArrayList<INCPanelModelListener> myListeners; 
	
	/**
	 * Specifies if the system has only one system root.
	 */
	private boolean myHasOneSystemRoot;
		
	/**
	 * Main constructor.
	 * @param model file system model
	 */
	public NCPanelModel(IContentModel model) {
		
		myLogger = Logger.getLogger("org.isakiev.fileManager");
		myModel = model;
		
		myTreeRoot = myModel.getRoot();
		myRootDirs = myModel.getChildren(myTreeRoot);
		
		myLogger.info("Number of root directories in the file system = " + myRootDirs.size());
		
		if (myRootDirs.size() == 0) {
			myLogger.severe("Root directories are not found!");
			throw new FileManagerException("Root directories are not found!");
		} else if (myRootDirs.size() == 1) {
			myHasOneSystemRoot = true;
			setCurrentDir((IDirEntity) myRootDirs.get(0));
		} else {
			myHasOneSystemRoot = false;
			setCurrentDir((IDirEntity) myRootDirs.get(1));
		}
		
		myRowsNumber = 18;
		myColumnsNumber = 2;
		myShift = 0;
		
		mySelectedRow = 0;
		mySelectedColumn = 0;
		
		myListeners = new ArrayList<INCPanelModelListener>(); 
	}
	
	
	/**
	 * Gets number of columns in the table
	 * @return number of columns
	 */
	public int getColumnsNumber() {
		return myColumnsNumber;
	}
	
	/**
	 * Gets number of rows in the table
	 * @return number of rows
	 */
	public int getRowsNumber() {
		return myRowsNumber;
	}
	
	/**
	 * Changes table structure according to new number of rows.
	 * @param newRowsNumber new number of rows
	 */
	public void setRowsNumber(int newRowsNumber) {
		if (newRowsNumber <= 0 ) {
			String message = "Wrong argument for setRowsNumber function " + "(newRowsNumber = " + newRowsNumber + ")";
			myLogger.severe(message);
			//throw new IllegalArgumentException(message);
			return; 
		}
		int selectedElementIndex = getIndexByPosition(mySelectedRow, mySelectedColumn);
		myRowsNumber = newRowsNumber;
		scrollToIndex(selectedElementIndex);
		firePanelDataChanged();
	}
	
	/**
	 * Returns number of files in current directory.
	 * @return number of files
	 */
	public int getNumberOfFilesInCurrentDirectory() {
		return myNumberOfFilesInCurrentDirectory;
	}
	
	/**
	 * Returns total size of the files in current directory.
	 * @return total size
	 */
	public long getSizeOfFilesInCurrentDirectory() {
		return mySizeOfFilesInCurrentDirectory;
	}
	
	/**
	 * Returns current directory entity
	 * @return current directory entity
	 */
	public IEntity getCurrentDirectoryEntity() {
		if (myLogger.isLoggable(Level.FINEST)) {
			myLogger.finest("Current directory: \"" + myCurrentDir.getFullName() + "\"");
		}
		return myCurrentDir;
	}
	
	/**
	 * Returns selected entity or <code>null</code> if nothing is selected
	 * @return selected entity or <code>null</code> if nothing is selected
	 */
	public IEntity getSelectedEntity() {
		checkSelection();
		int index = getIndexByPosition(mySelectedRow, mySelectedColumn);
		
		if ((index < 0) || (index > myCurrentChildren.size() - 1)) {
			if (myLogger.isLoggable(Level.FINEST)) {
				myLogger.finest("Nothing is selected (index = " + index + ").  Returning null.");
			}
			return null;
		} else {
			IEntity selectedEntity = myCurrentChildren.get(index);
			if (myLogger.isLoggable(Level.FINEST)) {
				myLogger.finest("Selected entity (index = " + index + "): \"" + selectedEntity.getFullName() + "\"");
			}
			return selectedEntity;
		}
	}
	
	/**
	 * Checks if cell with <code>row</code> and <code>column</code> coordinates is selected.
	 * @param row row 
	 * @param column column
	 * @return result of the check
	 */
	public boolean isSelected(int row, int column) {
		return (row == mySelectedRow) && (column == mySelectedColumn);
	}
	
	/**
	 * Checks if cell with <code>row</code> and <code>column</code> 
	 * coordinates is empty.
	 * @param row row 
	 * @param column column
	 * @return result of the check
	 */
	public boolean isEmpty(int row, int column) {
		int index = getIndexByPosition(row, column);
		return (index > myCurrentChildren.size() - 1) || (index < 0);
		
	}
	
	/**
	 * Checks if cell with <code>row</code> and <code>column</code> 
	 * coordinates is an upper level marker.
	 * @param row row 
	 * @param column column
	 * @return result of the check
	 */
	public boolean isUpperLevelMarker(int row, int column) {
		return myCanGoToUpperDir && (myShift == 0) && (row == 0) && (column == 0);
	}
	
	/**
	 * Gets element in the specified position.
	 * if position is empty.
	 * @param row row
	 * @param column column
	 * @return element in the specified position or <code>null</code> 
	 * if position is empty.
	 */
	public IEntity getElementAt(int row, int column) {
		if (isEmpty(row, column) || isUpperLevelMarker(row, column)) {
			return null;
		}
		int index = getIndexByPosition(row, column);
		return myCurrentChildren.get(index);
	}
	
	/**
	 * Checks if panel model can process enter action itself.
	 * For example move to the upper directory or to the subdirectory.
	 * @return result of the check.
	 */
	public boolean canProcessEnterAction() {
		if (isUpperLevelMarker(mySelectedRow, mySelectedColumn)) {
			return true;
		}
		IEntity entity = getSelectedEntity();
		if (entity == null) {
			return true;
		}
		if (entity.hasChildren()) {
			return true;
		}
		return false;
	}
	
	/**
	 * Gets entity index in current children list
	 * @param entity entity to find
	 * @return index in current children list or -1 if entity wasn't found
	 */
	private int getEntityIndex(IEntity entity) {
		int index = -1;
		for (IEntity e:myCurrentChildren) {
			index++;
			if (e.equals(entity)) {
				return index;
			}
		}
		return -1;
	}
	
	/**
	 * Processes enter actions if it is possible.
	 */
	public void processEnterAction() {
		if (isUpperLevelMarker(mySelectedRow, mySelectedColumn)) {
			IEntity toSelect = myCurrentDir;
			setCurrentDir((IDirEntity) (myCurrentDir.getParent()));
			scrollToIndex(getEntityIndex(toSelect));
			
		} else {
			IEntity entity = getSelectedEntity();
			if (entity != null) {
				if (entity.hasChildren()) {
					setCurrentDir((IDirEntity) entity);
					mySelectedRow = 0;
					mySelectedColumn = 0;
					myShift = 0;
				}
			}
		}
		firePanelDataChanged();
	}
	
	/**
	 * Sets current element. 
	 * @param directory new entity which will be set as current directory.
	 * @param selectedEntity element to select
	 */
	public void setCurrentElement(IDirEntity directory, IEntity selectedEntity) {
		if (myLogger.isLoggable(Level.FINE)) {
			myLogger.fine("Setting current element: \n directory = \"" + 
					(directory == null ? "null" : directory.getFullName()) + "\"\nselected entity = \"" +
					(selectedEntity == null ? "null" : selectedEntity.getFullName()) + "\"");
		}
		
		setCurrentDir(directory);
		if (selectedEntity == null) {
			myShift = 0;
			mySelectedRow = 0;
			mySelectedColumn = 0;
		} else {
			int index = -1;
			for (IEntity entity:myCurrentChildren) {
				index++;
				if (entity.equals(selectedEntity)) {
					break;
				}
			}
			scrollToIndex(index);
		}
		firePanelDataChanged();
	}
	

	/**
	 * Moves selection to the specified direction if possible.
	 * @param direction KeyEvent.VK_UP, KeyEvent.VK_DOWN, KeyEvent.VK_LEFT or KeyEvent.VK_RIGHT
	 */
	public void moveSelection(int direction) {
		
		int difference = 0;

		switch (direction) {
			case KeyEvent.VK_UP: {
				difference = -1;
				break;
			}
			case KeyEvent.VK_DOWN: {
				difference = 1;
				break;
			}
			case KeyEvent.VK_LEFT: {
				difference = -myRowsNumber;
				break;
			}
			case KeyEvent.VK_RIGHT: {
				difference = myRowsNumber;
				break;
			}
		}
		
		int index = getIndexByPosition(mySelectedRow, mySelectedColumn);
		int minValue = (myCanGoToUpperDir ? -1 : 0);
		int value = index + difference;
		if (value < minValue) {
			value = minValue;
		}
		if (value > myCurrentChildren.size() - 1) {
			value = myCurrentChildren.size() - 1;
		}
		scrollToIndex(value);
		
	}
	
	/**
	 * Moves selection to the specified position if it is correct.
	 * @param row row to select
	 * @param column column to select
	 */
	public void moveSelection(int row, int column) {
		int index = getIndexByPosition(row, column);
		if ((index >= 0 && index < myCurrentChildren.size()) 
			|| isUpperLevelMarker(row, column)) {
			
			mySelectedRow = row;
			mySelectedColumn = column;
			firePanelDataChanged();
		}
	}
	
	/**
	 * Sets file system tree root as a current directory.
	 */
	public void setRootAsCurrentDirectory() {
		
		//if there is one root directory
		if (myHasOneSystemRoot) {
			List<IEntity> children = myModel.getChildren(myTreeRoot);
			if (children.size() == 0) {
				myLogger.severe("Root directories were not found!");
			} else {
				setCurrentDir((IDirEntity) myCurrentChildren.get(0));
			}
		} else {
			setCurrentDir(myTreeRoot);
			mySelectedRow = 0;
			mySelectedColumn = 0;
		}
		checkSelection();
		firePanelDataChanged();
	}
	
	/**
	 * Adds listener.
	 * @param listener listener
	 */
	public void addPanelModelListener(INCPanelModelListener listener) {
		myListeners.add(listener);
	}
	
	/**
	 * Removes listener
	 * @param listener listener
	 */
	public void removePanelModelListener(INCPanelModelListener listener) {
		myListeners.remove(listener);
	}
	
	/**
	 * Checks if selected position points to an existing element and 
	 * corrects position if necessary.
	 */
	private void checkSelection() {
		int index = getIndexByPosition(mySelectedRow, mySelectedColumn);
		if ((index < 0)) {
			scrollToIndex(index);
		}
		if ((index > myCurrentChildren.size() - 1)) {
			scrollToIndex(myCurrentChildren.size() - 1);
		}
	}
	
	/**
	 * Transforms row and column numbers in current state into index 
	 * in the list of current directory subentities. 
	 * @param row row number
	 * @param column column number
	 * @return index in the list
	 */
	private int getIndexByPosition(int row, int column) {
		if (myCanGoToUpperDir) {
			return column * myRowsNumber + row + myShift - 1;
		} else {
			return column * myRowsNumber + row + myShift;
		}
	}
	
	
	/**
	 * Counts number of files in current directory and their size.
	 */
	private void countCurrentDirectoryStatistics() {
		myNumberOfFilesInCurrentDirectory = 0;
		mySizeOfFilesInCurrentDirectory = 0;
		for (IEntity entity: myCurrentChildren) {
			if (entity.isFileEntity()) {
				myNumberOfFilesInCurrentDirectory++;
				mySizeOfFilesInCurrentDirectory += entity.getSize();
			}
		}
	}
	
	/**
	 * Checks if there is a way from current directory to upper directory
	 * @return result of the check
	 */
	private boolean checkUpperDir() {
		IEntity parent = myCurrentDir.getParent();
		if (parent == null)
			return false;
		if (myHasOneSystemRoot && parent == myTreeRoot) {
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * Scrolls panel if index is out of visible area.
	 * @param index index if current enities list
	 */
	private void scrollToIndex(int index) {
		
		if (index < 0) {
			mySelectedRow = 0;
			mySelectedColumn = 0;
			myShift = 0;
		} else {
			
			//if index is in visible area, but it is necessary to decrease shift
			int i = getIndexByPosition(myRowsNumber - 1, myColumnsNumber - 1);
			if (myShift > 0 && i > myCurrentChildren.size()) {
				myShift = 0;
			}
			
			int sh = (myCanGoToUpperDir ? 1 : 0);
			
			if (index < myShift - sh) {
				//if index is above visible area
				myShift = index + sh;
				firePanelDataChanged();
			} else if (index >= myRowsNumber * myColumnsNumber + myShift - sh){
				//if index is below visible area
				myShift = index + sh + 1 - myRowsNumber * myColumnsNumber;
				firePanelDataChanged();
			} 
			
			int indexInVisibleArea = index - myShift + sh;
			
			mySelectedRow = indexInVisibleArea % myRowsNumber;
			mySelectedColumn = indexInVisibleArea / myRowsNumber;
		}
		firePanelDataChanged();
	}
	
	/**
	 * Sets specified directory as current
	 * @param currentDir directory
	 */
	private void setCurrentDir(IDirEntity currentDir) {
		if (currentDir == null) {
			myLogger.severe("Can't set current directory to null!");
			throw new FileManagerException("Can't set current directory to null!");
		}

		myCurrentDir = currentDir;
		myCurrentChildren = myModel.getChildren(myCurrentDir);
		myCanGoToUpperDir = checkUpperDir();
		countCurrentDirectoryStatistics();
		checkSelection();
		//firePanelDataChanged();
	}
	
	/**
	 * Informs all listeners about changes in panel model
	 */
	private void firePanelDataChanged() {
		EventObject event = new EventObject(this);
		if (myListeners != null) {
			for (INCPanelModelListener listener:myListeners) {
				listener.panelModelDataChanged(event);
			}
		}
	}
	
	/**
	 * Updates content of the current directory,
	 * sets new directory as current if current doesn't exist.
	 */
	public void updatePanelModel() {
		setCurrentDir((IDirEntity) (myModel.getEntityInsteadOfUnexisting(myCurrentDir)));
		checkSelection();
		firePanelDataChanged();
	}
	
	

}
