package org.isakiev.fileManager.contentViewers.nc;

import javax.swing.border.*;
import java.awt.*;


/**
 * This class is used to create borders with "Norton commander" style.
 * @author Isakiev Ruslan
 */
public class NCStyleBorderFactory {
	
	private static final int verticalInset = 22;
	private static final int horizontalInset = 10;
	
	/**
	 * Creates border with specified color and titles.
	 * @param color border and title color
	 * @param font title font
	 * @param topTitle top title
	 * @param bottomTitle bottom title
	 */
	public static Border createNCBorder(Color color, Font font, String topTitle, String bottomTitle) {
		Border borderWithoutBottom = new SpecialDoubleLineBorder(color, SpecialDoubleLineBorder.BOTTOM);
		Border borderWithoutTop = new SpecialDoubleLineBorder(color, SpecialDoubleLineBorder.TOP);
		MyTitledBorder topBorder = new MyTitledBorder(borderWithoutBottom, topTitle,  TitledBorder.TOP);
		MyTitledBorder bottomBorder = new MyTitledBorder(borderWithoutTop, bottomTitle,  TitledBorder.BOTTOM);
		topBorder.setTitleColor(color);
		topBorder.setTitleFont(font);
		bottomBorder.setTitleColor(color);
		bottomBorder.setTitleFont(font);
		Border border = new CompoundBorder(topBorder, bottomBorder);
		return new CompoundBorder(border, new EmptyBorder(verticalInset, horizontalInset, verticalInset, horizontalInset));
	}
	
	
	/**
	 * This is a titled border with zero insets.
	 */
	private static class MyTitledBorder extends TitledBorder {
		public MyTitledBorder(Border border, String title, int justification) {
			super(border, title, TitledBorder.CENTER, justification);
		}
		public Insets getBorderInsets(Component c) {
			return new Insets(0,0,0,0);
		}
	}
	
	/**
	 * This is a double line border without top or bottom part.
	 */
	private static class SpecialDoubleLineBorder implements Border {
		
		private Color myColor;
		private int myInvisibleBorder;
		
		public static final int TOP = 0; 
		public static final int BOTTOM = 1; 
		
		public SpecialDoubleLineBorder(Color color, int invisibleBorder) {
			myColor = color;
			myInvisibleBorder = invisibleBorder;
		}
		
		public Insets getBorderInsets(Component c) {
			return new Insets(0,0,0,0);
		}
		
		public boolean isBorderOpaque() {
			return true;
		}
		
		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
			
			int b = 2;               //space between lines
			int h = 2;               //horizontal shift
			int v = verticalInset;   //vertical shift
			
			//save previous color
			Color prevColor = g.getColor();
			g.setColor(myColor);
			if (myInvisibleBorder == BOTTOM) {
				//WITHOUT BOTTOM BORDER
				//draw top border
				g.drawLine(x + h, y, x + width - h, y);
				g.drawLine(x + h + b, y + b, x + width - h - b, y + b);
				//draw left border
				g.drawLine(x + h, y, x + h, y + height - v);
				g.drawLine(x + h + b, y + b, x + h + b, y + height - v);
				//draw right border
				g.drawLine(x + width - h, y, x + width - h, y + height - v);
				g.drawLine(x + width - h - b, y + b, x + width - h - b, y + height - v);
			} else {
				//WITHOUT TOP BORDER
				//draw bottom border
				g.drawLine(x + h, y + height, x + width - h, y + height);
				g.drawLine(x + h + b, y + height - b, x + width - h - b, y + height - b);
				//draw left border
				g.drawLine(x + h, y + v, x + h, y + height);
				g.drawLine(x + h + b, y + v, x + h + b, y + height - b);
				//draw right border
				g.drawLine(x + width - h, y + v, x + width - h, y + height);
				g.drawLine(x + width - h - b, y + v, x + width - h - b, y + height - b);
			}

			//restore color
			g.setColor(prevColor);
		}
	}
	
	public static Border createDoubleLineBorder(Color color) {
		Border border1 = new CompoundBorder(new EmptyBorder(4,2,4,4), new LineBorder(color));
		Border border2 = new CompoundBorder(new EmptyBorder(1,1,1,1), new LineBorder(color));
		Border border3 = new CompoundBorder(border1,border2);
		return border3; //new CompoundBorder(border3, new EmptyBorder(15,2,15,2));
	}
	

}
