package org.isakiev.fileManager.entities;

import java.io.File;

import org.isakiev.fileManager.entityTypes.IEntityType;

/**
 * Basic class, a tree node, representing file system entity (directory, file etc.)
 * @author Isakiev Ruslan.
 */
abstract class Entity implements IEntity {
	
	private File myFile;
	private IEntity myParent;
	private IEntityType myType;
	private boolean myIsTreeRoot;
	
	private String myFileName;
	private long myFileSize;
	private long myLastModified;
	
	/**
	 * Main constructor
	 * @param file file or directory pathname
	 * @param parent parent in the tree
	 */
	public Entity(File file, IEntity parent, IEntityType type) {
		myFile = file;
		myParent = parent;
		myType = type;
		myIsTreeRoot = false;
		
		myFileName = file.getName();
		myFileSize = (isFileEntity() ? file.length() : 0);
		myLastModified = (isDiskEntity() ? 0 : file.lastModified());
	}
	
	/**
	 * Returns <code>File</code> assigned to this entity. 
	 * @return <code>File</code> assigned to this entity.
	 */
	public File getFile() {
		return myFile;
	}
	
	/**
	 * Returns parent in the tree.
	 * @return parent in the tree.
	 */
	public IEntity getParent() {
		return myParent;
	}
	
	/**
	 * Returns the type of current class
	 * @return the type of current class
	 */
	public IEntityType getType() {
		return myType;
	}
	
	/**
	 * Marks an entity as a global root.
	 */
	public void markAsTreeRoot(){
		myIsTreeRoot = true;
	}	
	
	/**
	 * Returns short entity name. It displayed in viewers.
	 * @return short entity name
	 */
	public String getShortName() {
		return myFileName; 
	}
	
	/**
	 * Returns full entity name.
	 * @return full entity name.
	 */
	public String getFullName() {
		return myFile.getAbsolutePath();
	}
	
	/**
	 * Returns date when entity was last modified.
	 * @return date when entity was last modified.
	 */
	public long getLastModified() {
		return myLastModified;
	}
	
	/**
	 * Returns entity size. 
	 * @return 0 if entity is a directory and entity size otherwise.
	 */
	public long getSize() {
		return myFileSize; //(isFileEntity() ? myFile.length() : 0);
	}
	
	
	/**
	 * Returns string representation of an entity.
	 * @return string representation of an entity.
	 */
	public String toString() {
		if (myIsTreeRoot) {
			return "File system";
		} else {
			return getShortName();
		}
	}
	

	/**
	 * Returns <code>true</code> if entity is an archive
	 * @return <code>true</code> if entity is an archive
	 */
	public boolean isArchiveEntity() {
		return false;
	}
	
	/**
	 * Returns <code>true</code> if entity is compressed entity
	 * @return <code>true</code> if entity is compressed entity
	 */
	public boolean isCompressedEntity() {
		return false;
	}
	
	/**
	 * Checks if the Entity is FileEntity
	 * @return check result 
	 */
	public boolean isFileEntity() {
		return false;
	}
	
	/**
	 * Checks if the Entity is DirEntity
	 * @return check result
	 */
	public boolean isDirEntity() {
		return false;
	}
	
	/**
	 * Checks if the Entity is DiskEntity
	 * @return check result
	 */
	public boolean isDiskEntity() {
		return false;
	}
	
	/**
	 * Checks if Entity has children.
	 * @return check result
	 */
	public boolean hasChildren() {
		return false;
	}
	
	/**
	 * Checks if entity is the file system's tree root.
	 * @return result of the check;
	 */
	public boolean isTreeRoot() {
		return myIsTreeRoot;
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof IEntity) {
			IEntity entity = (IEntity) obj;
			return entity.getFullName().equals(getFullName());
		}
		return false;
	}

}
