package org.isakiev.fileManager.tests;

import junit.framework.TestCase;
import java.io.*;
import java.util.*;

import org.isakiev.fileManager.contentManagers.*;
import org.isakiev.fileManager.contentModel.*;
import org.isakiev.fileManager.entities.*;



/**
 * Tests for operations.
 * @author Isakiev Ruslan
 */
public class TestOperations extends TestCase {
	private static String myOperationsDir = "4tests/operations/";
	private IContentManager myManager; 
	private IContentModel myModel; 
	
	private ArrayList<String> myExampleDirectoryNames;
	private ArrayList<String> myExampleFileNames;
	
	public TestOperations() {
		myManager = new ContentManager();
		myModel = new ContentModel(myManager);
		File f = new File(myOperationsDir);
		if (!f.exists()) {
			f.mkdirs();
		}
		
		myExampleDirectoryNames = new ArrayList<String>();
		myExampleDirectoryNames.add("dir1");
		myExampleDirectoryNames.add("dir1/subdir11");
		myExampleDirectoryNames.add("dir2");
		myExampleDirectoryNames.add("dir3");
		myExampleDirectoryNames.add("dir3/subdir31");
		myExampleDirectoryNames.add("dir3/subdir32");
		myExampleDirectoryNames.add("dir3/subdir33");
		myExampleDirectoryNames.add("dir4");
		
		myExampleFileNames = new ArrayList<String>();
		myExampleFileNames.add("dir1/file1.file");
		myExampleFileNames.add("dir1/file2.file");
		myExampleFileNames.add("dir1/subdir11/file111.file");
		myExampleFileNames.add("dir1/subdir11/file112.file");
		myExampleFileNames.add("dir2/file21.file");
		myExampleFileNames.add("dir2/file22.file");
		myExampleFileNames.add("dir2/file23.file");
		myExampleFileNames.add("dir3/subdir32/file1.file");
		myExampleFileNames.add("dir3/file31.file");
		myExampleFileNames.add("file.file");
		
		//sort directories by length 
		Collections.sort(myExampleDirectoryNames, new Comparator<String>() {
			public int compare(String s1, String s2) {
				return new Integer(s2.length()).compareTo(s1.length());
			}
		});
	}
	
	/**
	 * Creates examples
	 * @param examplesDir examples' directory
	 */
	private void createExamples(String examplesDir) {
		for (String name: myExampleDirectoryNames) {
			File f = new File(examplesDir + name);
			f.mkdirs();
		}
		int i = 0;
		for (String name: myExampleFileNames) {
			i++;
			Integer[] bytes = new Integer[i];
			for (int k=0; k<i; k++) {
				bytes[k] = k;
			}
			ExamplesCreator.createFile(examplesDir + name, bytes);
		}
	}
	
	/**
	 * Deletes examples
	 * @param examplesDir examples' directory
	 */
	private void checkExamples(String examplesDir) {
		int i = 0;
		for (String name: myExampleFileNames) {
			i++;
			File file = new File(examplesDir + name);
			if (!file.exists()) {
				fail("Required file \"" + file.getAbsolutePath() + "\" doesn't exist");
			} else {
				if (file.length() != i) {
					fail("Wrong length of file \"" + file.getAbsolutePath() + "\"!");
				}
			}
		}
	}
	
	/**
	 * Gets from the list entity with the specified short name.
	 * @param list list with entities
	 * @param name name
	 * @return entity with the specified short name or <code>null</code> if entitywasn't found
	 */
	private IEntity getEntityByName(List<IEntity> list, String name) {
		for (IEntity entity:list) {
			if (entity.getShortName().equals(name)) {
				return entity;
			}
		}
		return null;
	}
	
	private void checkIfEntityIsPresent(IDirEntity dir, String entityName) {
		String[] pathComponents = entityName.split("/");
		String currentPath = "";
		IDirEntity currentDir = dir;
		for (int i=0; i<pathComponents.length; i++) {
			currentPath += ("/" + pathComponents[i]);
			IEntity e = getEntityByName(myModel.getChildren(currentDir), pathComponents[i]);
			if (e == null) {
				fail("\"" + currentPath + "\" wasn't found!");
			}
			if (i < pathComponents.length - 1) {
				if (!e.hasChildren()) {
					fail("\"" + currentPath + "\" has no children!");
				} else {
					currentDir = (IDirEntity) e;
				}
			}
		}
	}
	
	
	/**
	 * Checks if information in the model is correct.
	 * @param examplesDirEntity
	 */
	private void checkExamplesInContentModel(IDirEntity examplesDirEntity) {
		
		for (String name: myExampleDirectoryNames) {
			checkIfEntityIsPresent(examplesDirEntity, name);
		}
		
		for (String name: myExampleFileNames) {
			checkIfEntityIsPresent(examplesDirEntity, name);
		}
		
	}
	
	/**
	 * Checks if files in specified directory are equal to examples
	 * @param examplesDir directory to check
	 */
	private void deleteExamples(String examplesDir) {
		for (String name: myExampleFileNames) {
			File file = new File(examplesDir + name);
			file.delete();
		}
		for (String name: myExampleDirectoryNames) {
			File file = new File(examplesDir + name);
			file.delete();
		}
		File file = new File(examplesDir);
		file.delete();
	}
	
	/**
	 * Tests if directories are created.
	 */
	public void testDirectoryCreation() {
		File f = new File(myOperationsDir);
		IDirEntity dir = EntityFactory.createDirEntity(f, null, null);
		String name = "testDir";
		myManager.createDirectory(dir, name, processInfoListener);
		f = new File(myOperationsDir + name);
		if (!f.exists()) {
			fail("Directory creation failed!");
		} else {
			f.delete();
		}
	}
	
	public void testDeletingEmptyDirectory() {
		File f = new File(myOperationsDir + "someDir");
		if (!f.mkdir() && !f.exists()) {
			fail("[preparing for test] Can't create directory for test!");
		}
		IEntity entity = EntityFactory.createDirEntity(f, null, null);
		myManager.delete(entity, processInfoListener);
		if (f.exists()) {
			fail("Empty directory deleting failed!");
		}
	}
	
	public void testDeletingNotEmptyDirectory() {
		createExamples(myOperationsDir + "examples/");
		File f = new File(myOperationsDir + "examples");
		IEntity entity = EntityFactory.createDirEntity(f, null, null);
		myManager.delete(entity, processInfoListener);
		if (f.exists()) {
			deleteExamples(myOperationsDir + "examples/");
			fail("Empty directory deleting failed!");
		}
	}
	
	public void testCopyActions() {
		File mainDir = new File(myOperationsDir);
		createExamples(myOperationsDir + "source/");
		File srcFile = new File(mainDir, "source");
		File destFile = new File(mainDir, "destination");
		IEntity entity = EntityFactory.createDirEntity(srcFile, null, null);
		myManager.copy(entity, srcFile.getAbsolutePath(), destFile.getAbsolutePath(), processInfoListener);
		
		checkExamples(myOperationsDir + "destination/");
		
		deleteExamples(myOperationsDir + "source/");
		deleteExamples(myOperationsDir + "destination/");
	}
	
	public void testCopyActionsInModel() {
		File mainDir = new File(myOperationsDir);
		createExamples(myOperationsDir + "source/");
		File srcFile = new File(mainDir, "source");
		File destFile = new File(mainDir, "destination");
		IEntity srcEntity = EntityFactory.createDirEntity(srcFile, null, null);
		myManager.copy(srcEntity, srcFile.getAbsolutePath(), destFile.getAbsolutePath(), processInfoListener);
		
		IEntity destEntity = myModel.getEntity(destFile.getAbsolutePath());
		checkExamplesInContentModel((IDirEntity) destEntity);
		
		deleteExamples(myOperationsDir + "source/");
		deleteExamples(myOperationsDir + "destination/");
	}
	
	public void testRenameActions() {
		File mainDir = new File(myOperationsDir);
		createExamples(myOperationsDir + "examples/");
		File file = new File(mainDir, "examples");

		IEntity entity = EntityFactory.createDirEntity(file, null, null);
		String newName = mainDir.getAbsolutePath() + File.separator + "renamed";
		myManager.renMove(entity, newName, processInfoListener);
				
		checkExamples(myOperationsDir + "renamed/");
		deleteExamples(myOperationsDir + "renamed/");
	}
	
	public void testRenameActionsInModel() {
		File mainDir = new File(myOperationsDir);
		createExamples(myOperationsDir + "examples/");
		File file = new File(mainDir, "examples");

		IEntity entity = EntityFactory.createDirEntity(file, null, null);
		String newName = mainDir.getAbsolutePath() + File.separator + "renamed";
		myManager.renMove(entity, newName, processInfoListener);
				
		IEntity destEntity = myModel.getEntity(newName);
		checkExamplesInContentModel((IDirEntity) destEntity);
		deleteExamples(myOperationsDir + "renamed/");
	}
	
	public void testMoveActions() {
		File mainDir = new File(myOperationsDir);
		createExamples(myOperationsDir + "examples/");
		File file = new File(mainDir, "examples");
		IEntity entity = EntityFactory.createDirEntity(file, null, null);
		
		File dir = new File(myOperationsDir + "somedir");
		dir.mkdir();
		String newName = mainDir.getAbsolutePath() + File.separator + "somedir" + File.separator + "folder";
		myManager.renMove(entity, newName, processInfoListener);
		
		checkExamples(myOperationsDir + "somedir/folder/");
		
		deleteExamples(myOperationsDir + "somedir/folder/");
		dir.delete();
	}
	
	public void testMoveActionsInModel() {
		File mainDir = new File(myOperationsDir);
		createExamples(myOperationsDir + "examples/");
		File file = new File(mainDir, "examples");
		IEntity entity = EntityFactory.createDirEntity(file, null, null);
		
		File dir = new File(myOperationsDir + "somedir");
		dir.mkdir();
		String newName = mainDir.getAbsolutePath() + File.separator + "somedir" + File.separator + "folder";
		myManager.renMove(entity, newName, processInfoListener);
		
		IEntity destEntity = myModel.getEntity(newName);
		checkExamplesInContentModel((IDirEntity) destEntity);
		
		deleteExamples(myOperationsDir + "somedir/folder/");
		dir.delete();
	}
	
	
	
	
	
	private IProcessInfoListener processInfoListener = new IProcessInfoListener() {
		public void progressChanged(int percents) { /*do nothing */ }
		public void processingNewFile(String name) { /*do nothing */ }
		public void showMessage(String message) { 
			System.out.println(message);
		}
	};
	
}
