package org.isakiev.fileManager.utils;

import java.util.*;

import java.util.logging.*;
import org.w3c.dom.*;

/**
 * Loads information about features, version history and statistics from XML.
 * @author Ruslan Isakiev
 */

public class InfoLoader {
	
	private static InfoLoader myInstance = null;
	
	private final String myVersionHistoryPath = "data/VersionHistory.xml";
	private final String myFeaturesListPath = "data/FeaturesList.xml";
	
	private final String myVersionHistoryName = "versionHistory";
	private final String myReleaseName = "release";
	private final String myVersionName = "version";
	private final String myDateName = "date";
	private final String myJavaFilesName = "javaFiles";
	private final String myClassFilesName = "classFiles";
	private final String myCodeSizeName = "codeSize";
	
	private final String myFeatureName = "feature";
	private final String myDescriptionName = "description";
	private final String myFeaturesListName = "featuresList";
	
	private Logger myLogger;
	
	/**
	 * Information about version of the FileManager
	 */
	public class Info {
		public String version;
		public String date;
		public int javaFiles;
		public int classFiles;
		public int codeSize;
	}
	
	private ArrayList<Info> myRecords;
	private ArrayList<String> myFeatures;
	
	public static InfoLoader getInstance() {
		if (myInstance == null) {
			myInstance = new InfoLoader();
		}
		return myInstance;
	}
	
	public List<Info> getInfo() {
		return Collections.unmodifiableList(myRecords);
	}
	
	public List<String> getFeatures() {
		return Collections.unmodifiableList(myFeatures);
	}
	
	/**
	 * Loads information from XML files.
	 */
	private InfoLoader() {
		myLogger = Logger.getLogger("org.isakiev.fileManager");
		myRecords = new ArrayList<Info>();
		Document document = Tools.getTools().loadXMLDocument(myVersionHistoryPath);
		if (document == null) {
			myLogger.warning("Version history wasn't loaded!");
		} else {
			Element root = document.getDocumentElement();
			NodeList list = root.getElementsByTagName(myReleaseName);
			if (list!=null) {
				for (int i=0; i<list.getLength(); i++) {
					Node release = list.item(i);
					Info info = new Info();
					info.version = getStringAttribute(release, myVersionName, "unknown");
					info.date = getStringAttribute(release, myDateName, "unknown");
					info.javaFiles = getIntegerAttribute(release, myJavaFilesName, -1);
					info.classFiles = getIntegerAttribute(release, myClassFilesName, -1);
					info.codeSize = getIntegerAttribute(release, myCodeSizeName, -1);
					myRecords.add(info);
				}
			}
			
		}
		
		myFeatures = new ArrayList<String>();
		document = Tools.getTools().loadXMLDocument(myFeaturesListPath);
		if (document == null) {
			myLogger.warning("Features list wasn't loaded!");
		} else {
			Element root = document.getDocumentElement();
			NodeList list = root.getElementsByTagName(myFeatureName);
			if (list!=null) {
				for (int i=0; i<list.getLength(); i++) {
					Node feature = list.item(i);
					String description = getStringAttribute(feature, myDescriptionName, "unknown");
					myFeatures.add(description);
				}
			}
		}
		
	}
	
	
	/**
	 * Gets an attribute with specified name from the node
	 * @param node node
	 * @param attrName attribute name
	 * @param defaultValue default value
	 * @return attribute value or <code>defaultValue</code> if attribute not found.
	 */
	private int getIntegerAttribute(Node node, String attrName, int defaultValue) {
		Node item = node.getAttributes().getNamedItem(attrName);
		if (item != null) {
			int value;
			try {
				value = Integer.parseInt(item.getNodeValue());
			} catch (NumberFormatException nfe) {
				myLogger.warning("attribute \"" + attrName	+ "\" has wrong number format!");
				return defaultValue;
			}
			return value;
		} else {
			myLogger.warning("attribute \"" + attrName + "\" wasn't found in node \"" + node.getNodeName() + "\"!");
			return defaultValue;
		}
	}
	
	/**
	 * Gets an attribute with specified name from the node
	 * @param node node
	 * @param attrName attribute name
	 * @param defaultValue default value
	 * @return attribute value or <code>defaultValue</code> if attribute not found.
	 */
	private String getStringAttribute(Node node, String attrName, String defaultValue) {
		Node item = node.getAttributes().getNamedItem(attrName);
		if (item != null) {
			return item.getNodeValue();
		} else {
			myLogger.warning("attribute \"" + attrName + "\" wasn't found in node \"" + node.getNodeName() + "\"!");
			return defaultValue;
		}
	}
}
