package org.isakiev.fileManager.utils;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.isakiev.fileManager.contentViewers.SupportedViewerStyles;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.awt.Point;
import java.awt.Dimension;

import java.util.logging.*;

/**
 * This class provides load and save operations for current state information.
 * @author Isakiev Ruslan
 */
public class StateLoader {
	
	private static StateLoader myInstance = null;
	
	private final String myDocumentRootName = "fileManagerState";
	
	private final String myFrameStateName = "frameState";
	private final String myWidthName = "width";
	private final String myHeightName = "height";
	private final String myLocationXName = "locationX";
	private final String myLocationYName = "locationY";
	private final String myStyleName = "style";
	private final String myNCStyleName = "nc";
	private final String myExplorerStyleName = "explorer";
	
	private final String myNCStateName = "ncState";
	private final String myLeftEntityName = "leftEntity";
	private final String myRightEntityName = "rightEntity";
	private final String myLeftDirectoryName = "leftDirectory";
	private final String myRightDirectoryName = "rightDirectory";
	private final String myNullName = "null";
	private final String myActivePanelIndexName = "activePanelIndex";
	
	private final String myExplorerStateName = "explorerState";
	private final String myCurrentEntityName = "currentEntity";
	private final String myCurrentDirectoryName = "currentDirectory";
	private final String myDividerLocationName = "dividerLocation";
	
	private int myWidth;
	private int myHeight;
	private int myLocationX;
	private int myLocationY;
	private SupportedViewerStyles myStyle;
	
	private String myLeftEntityString; 
	private String myRightEntityString; 
	private String myLeftDirectoryString; 
	private String myRightDirectoryString; 
	private int myActivePanelIndex;
	
	private String myCurrentEntityString; 
	private String myCurrentDirectoryString; 
	private int myDividerLocation;
	
	//private static final String USER_HOME = System.getProperty("user.home");
	private static final String USER_DIR = System.getProperty("user.dir");
	private Logger myLogger;
	
	/**
	 * Restores dafault values.
	 */
	private void restoreDefaults() {
		myWidth = 800;
		myHeight = 505;
		myLocationX = 100;
		myLocationY = 100;
		myStyle = SupportedViewerStyles.NC_STYLE;
		
		myLeftDirectoryString = USER_DIR; 
		myRightDirectoryString = USER_DIR; 
		myLeftEntityString = myNullName; 
		myRightEntityString = myNullName; 
		myActivePanelIndex = 0;
		
		myCurrentDirectoryString = USER_DIR; 
		myCurrentEntityString = myNullName; 
		myDividerLocation = -1;
	}
	
	/**
	 * Returns StateManager's instance
	 * @return StateManager's instance
	 */
	public static StateLoader getInstance() {
		if (myInstance == null) {
			myInstance = new StateLoader();
		}
		return myInstance;
	}
	
	/**
	 * Main constructor
	 */
	private StateLoader() {
		restoreDefaults();
		myLogger = Logger.getLogger("org.isakiev.fileManager");
	}
	
	/**
	 * Saves current file manager's state.
	 * @param fileName file name
	 */
	public void saveState(String fileName) {
		Document document;
		try {
			//create document
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setNamespaceAware(true);
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.newDocument();
			Element root = document.createElement(myDocumentRootName);
			
			//add main frame info 
			Element frameState = document.createElement(myFrameStateName);
			frameState.setAttribute(myLocationXName, new Integer(myLocationX).toString());
			frameState.setAttribute(myLocationYName, new Integer(myLocationY).toString());
			frameState.setAttribute(myHeightName, new Integer(myHeight).toString());
			frameState.setAttribute(myWidthName, new Integer(myWidth).toString());
			String style = (myStyle == SupportedViewerStyles.NC_STYLE ? myNCStyleName : myExplorerStyleName);
			frameState.setAttribute(myStyleName,style);
			root.appendChild(frameState);
			
			//add NC style viewer info
			Element ncState = document.createElement(myNCStateName);

			String leftEntity = (myLeftEntityString != null ? myLeftEntityString : myNullName);
			String rightEntity = (myRightEntityString != null ? myRightEntityString : myNullName);
			String leftDirectory = (myLeftDirectoryString != null ? myLeftDirectoryString : myNullName);
			String rightDirectory = (myRightDirectoryString != null ? myRightDirectoryString : myNullName);
			ncState.setAttribute(myLeftEntityName, leftEntity);
			ncState.setAttribute(myRightEntityName, rightEntity);
			ncState.setAttribute(myLeftDirectoryName, leftDirectory);
			ncState.setAttribute(myRightDirectoryName, rightDirectory);
			
			ncState.setAttribute(myActivePanelIndexName, new Integer(myActivePanelIndex).toString());
			root.appendChild(ncState);
			
			//add explorer style viewer info
			Element explorerState = document.createElement(myExplorerStateName);
			
			String currentEntity = (myCurrentEntityString != null ? myCurrentEntityString : myNullName);
			String currentDirectory = (myCurrentDirectoryString != null ? myCurrentDirectoryString : myNullName);
			explorerState.setAttribute(myCurrentEntityName, currentEntity);
			explorerState.setAttribute(myCurrentDirectoryName, currentDirectory);
			explorerState.setAttribute(myDividerLocationName, new Integer(myDividerLocation).toString());
			root.appendChild(explorerState);
			
			//write document
			document.appendChild(root);
			Tools.getTools().saveXMLDocument(document, fileName);
			if (myLogger.isLoggable(Level.INFO)) {
				myLogger.info(createSettingsList("Current state was successfully saved to \"" + fileName + "\"\nSettings: "));
			}

		} catch (ParserConfigurationException pce) {
			myLogger.log(Level.SEVERE, "Cannot create parser!", pce);
			return;
		} catch (DOMException de) {
			myLogger.log(Level.SEVERE, "DOM exception!", de);
			return;
		}
	}
	
	/**
	 * Tries to load current filemanager's state. 
	 * @param fileName file name
	 */
	public void loadState(String fileName) {
		restoreDefaults();
		Document document = Tools.getTools().loadXMLDocumentFromFile(fileName);

		if (document == null) {
			myLogger.warning("State wasn't loaded!");
			return;
		}
		Element root = document.getDocumentElement();
		Node frameState = getTopLevelNode(root, myFrameStateName);
		if (frameState != null) {
			myWidth = getIntegerAttribute(frameState, myWidthName, myWidth);
			myHeight = getIntegerAttribute(frameState, myHeightName, myHeight);

			myLocationX = getIntegerAttribute(frameState, myLocationXName, myLocationX);
			myLocationY = getIntegerAttribute(frameState, myLocationYName, myLocationY);
			
			String style = getStringAttribute(frameState, myStyleName, myNCStyleName);
			if (style.equals(myNCStyleName)) {
				//myActionMap.get("switch to nc style").actionPerformed(null);
				myStyle = SupportedViewerStyles.NC_STYLE;
			} else {
				//myActionMap.get("switch to explorer style").actionPerformed(null);
				myStyle = SupportedViewerStyles.EXPLORER_STYLE;
			}
		}
		
		Node ncState = getTopLevelNode(root, myNCStateName);
		if (ncState != null) {
			myLeftEntityString = getStringAttribute(ncState, myLeftEntityName, myNullName);
			myRightEntityString = getStringAttribute(ncState, myRightEntityName, myNullName);
			myLeftDirectoryString = getStringAttribute(ncState, myLeftDirectoryName, USER_DIR);
			myRightDirectoryString = getStringAttribute(ncState, myRightDirectoryName, USER_DIR);
			myActivePanelIndex = getIntegerAttribute(ncState, myActivePanelIndexName, myActivePanelIndex);
		}
		
		Node explorerState = getTopLevelNode(root, myExplorerStateName);
		if (explorerState != null) {
			myCurrentEntityString = getStringAttribute(explorerState, myCurrentEntityName, myNullName);
			myCurrentDirectoryString = getStringAttribute(explorerState, myCurrentDirectoryName, USER_DIR);
			myDividerLocation = getIntegerAttribute(explorerState, myDividerLocationName, myDividerLocation);
		}
		
		if (myLogger.isLoggable(Level.INFO)) {
			myLogger.info(createSettingsList("Current state was loaded."));
		}
	}
	
	/**
	 * Creates settings list with specified title
	 * @param title title
	 * @return settings list with specified title
	 */
	private String createSettingsList(String title) {
		StringBuffer buff = new StringBuffer();
		buff.append(title).append("\n");
		
		buff.append(myLocationXName).append("=").append(myLocationX).append("; \n");
		buff.append(myLocationYName).append("=").append(myLocationY).append("; \n");
		buff.append(myHeightName).append("=").append(myHeight).append("; \n");
		buff.append(myWidthName).append("=").append(myWidth).append("; \n");
		buff.append(myStyleName).append("=").append(myStyle == SupportedViewerStyles.NC_STYLE ? myNCStyleName : myExplorerStyleName).append("; \n");
		
		buff.append(myLeftDirectoryName).append("=").append(myLeftDirectoryString != null ? myLeftDirectoryString : myNullName).append("; \n");
		buff.append(myLeftEntityName).append("=").append(myLeftEntityString != null ? myLeftEntityString : myNullName).append("; \n");
		buff.append(myRightDirectoryName).append("=").append(myRightDirectoryString != null ? myRightDirectoryString : myNullName).append("; \n");
		buff.append(myRightEntityName).append("=").append(myRightEntityString != null ? myRightEntityString : myNullName).append("; \n");
		buff.append(myActivePanelIndexName).append("=").append(myActivePanelIndex).append("; \n");
					
		buff.append(myCurrentEntityName).append("=").append(myCurrentEntityString != null ? myCurrentEntityString : myNullName).append("; \n");
		buff.append(myCurrentDirectoryName).append("=").append(myCurrentDirectoryString != null ? myCurrentDirectoryString : myNullName).append("; \n");
		buff.append(myDividerLocationName).append("=").append(myDividerLocation).append("; \n");
		
		//System.out.println(buff.toString());
		return buff.toString();
	}
	
	/**
	 * Returns first node with a specified tag name in the root element.
	 * @param root root element
	 * @param tagName tag name
	 * @return node
	 */
	private Node getTopLevelNode(Element root, String tagName) {
		NodeList list = root.getElementsByTagName(tagName);
		if (list != null) {
			if (list.getLength()>0) {
				return list.item(0);
			}
		}
		if (myLogger.isLoggable(Level.WARNING)) {
			myLogger.warning("node with \"" +  tagName+ "\" tag name wasn't found!");
		}
		return null;
	}
	
	/**
	 * Gets an attribute with specified name from the node
	 * @param node node
	 * @param attrName attribute name
	 * @param defaultValue default value
	 * @return attribute value or <code>defaultValue</code> if attribute not found.
	 */
	private int getIntegerAttribute(Node node, String attrName, int defaultValue) {
		Node item = node.getAttributes().getNamedItem(attrName);
		if (item != null) {
			int value;
			try {
				value = Integer.parseInt(item.getNodeValue());
			} catch (NumberFormatException nfe) {
				if (myLogger.isLoggable(Level.WARNING)) {
					myLogger.warning("attribute \"" + attrName+ "\" has wrong number format!");
				}
				return defaultValue;
			}
			return value;
		} else {
			if (myLogger.isLoggable(Level.WARNING)) {
				myLogger.warning("attribute \"" + attrName + "\" wasn't found in node \"" + node.getNodeName() + "\"!");
			}
			return defaultValue;
		}
	}
	
	/**
	 * Gets an attribute with specified name from the node
	 * @param node node
	 * @param attrName attribute name
	 * @param defaultValue default value
	 * @return attribute value or <code>defaultValue</code> if attribute not found.
	 */
	private String getStringAttribute(Node node, String attrName, String defaultValue) {
		Node item = node.getAttributes().getNamedItem(attrName);
		if (item != null) {
			return item.getNodeValue();
		} else {
			if (myLogger.isLoggable(Level.WARNING)) {
				myLogger.warning("attribute \"" + attrName + "\" wasn't found in node \"" + node.getNodeName() + "\"!");
			}
			return defaultValue;
		}
	}

	
	/**
	 * Sets frame size
	 * @param size frame size
	 */
	public void setSize(Dimension size) {
		myWidth = size.width;
		myHeight = size.height;
	}
	
	/**
	 * Gets frame size
	 * @return frame size
	 */
	public Dimension getSize() {
		return new Dimension(myWidth, myHeight);
	}
	
	/**
	 * Sets frame location
	 * @param location frame location
	 */
	public void setLocation(Point location) {
		myLocationX = location.x;
		myLocationY = location.y;
	}
	
	/**
	 * Gets frame location
	 * @return frame location
	 */
	public Point getLocation() {
		return new Point(myLocationX, myLocationY);
	}
	
	
	
	/**
	 * Sets viewer style
	 * @param style viewer style
	 */
	public void setStyle(SupportedViewerStyles style) {
		myStyle = style;
	}
	
	/**
	 * Gets viewer style
	 * @return viewer style
	 */
	public SupportedViewerStyles getStyle() {
		return myStyle;
	}
	
	/**
	 * Sets left entity's string for NC style viewer
	 * @param entityString left entity's string
	 */
	public void setLeftEntityString(String entityString) {
		myLeftEntityString = entityString;
	}
	
	/**
	 * Gets left entity's string for NC style viewer
	 * @return left entity's string. If left entity is undefined, null is returned.
	 */
	public String getLeftEntityString() {
		return myLeftEntityString.equals(myNullName) ? null : myLeftEntityString;
	}
	
	/**
	 * Sets left directory's string for NC style viewer
	 * @param dirString left directory's string
	 */
	public void setLeftDirectoryString(String dirString) {
		myLeftDirectoryString = (dirString == null ? USER_DIR : dirString);
	}
	
	/**
	 * Gets left directory's string for NC style viewer
	 * @return  left directory's string. If left directory is undefined, empty string is returned.
	 */
	public String getLeftDirectoryString() {
		return myLeftDirectoryString;
	}
	
	/**
	 * Sets right entity's string for NC style viewer
	 * @param entityString right entity's string
	 */
	public void setRightEntityString(String entityString) {
		myRightEntityString = entityString;
	}
	
	/**
	 * Gets right entity's string for NC style viewer
	 * @return right entity's string. If right entity is undefined, null is returned.
	 */
	public String getRightEntityString() {
		return myRightEntityString.equals(myNullName) ? null : myRightEntityString;
	}
	
	/**
	 * Sets right directory's string for NC style viewer
	 * @param dirString right directory's string
	 */
	public void setRightDirectoryString(String dirString) {
		myRightDirectoryString = (dirString == null ? USER_DIR : dirString);
	}
	
	/**
	 * Gets right directory's string for NC style viewer
	 * @return right directory's string. If right directory is undefined, empty string is returned.
	 */
	public String getRightDirectoryString() {
		return myRightDirectoryString;
	}
	
	/**
	 * Sets NC style viewer's active panel index
	 * @param activePanel active panel index
	 */
	public void setActivePanelIndex(int activePanelIndex) {
		myActivePanelIndex = activePanelIndex;
	}
	
	/**
	 * Gets NC style viewer's active panel index
	 * @return active panel index
	 */
	public int getActivePanelIndex() {
		return myActivePanelIndex;
	}
	
	/**
	 * Sets current entity's string for explorer style viewer
	 * @param entityString current entity's string
	 */
	public void setCurrentEntityString(String entityString) {
		myCurrentEntityString = entityString;
	}
	
	/**
	 * Gets current entity's string for explorer style viewer
	 * @return current entity's string. If right entity is undefined, null is returned.
	 */
	public String getCurrentEntityString() {
		return myCurrentEntityString;
	}
	
	/**
	 * Sets current directory's string for explorer style viewer
	 * @param dirString current entity's string
	 */
	public void setCurrentDirectoryString(String dirString) {
		myCurrentDirectoryString = (dirString == null ? USER_DIR : dirString);
	}
	
	/**
	 * Gets current directory's string for explorer style viewer
	 * @return current directory's string. 
	 */
	public String getCurrentDirectoryString() {
		return myCurrentDirectoryString;
	}
	
	/**
	 * Sets explorer style viewer JSplitPane's divider location
	 * @param dividerLocation divider location
	 */
	public void setDividerLocation(int dividerLocation) {
		myDividerLocation = dividerLocation;
	}
	
	/**
	 * Gets explorer style viewer JSplitPane's divider location
	 * @return divider location
	 */
	public int getDividerLocation() {
		return myDividerLocation;
	}
	
	

}
