package org.isakiev.fileManager.utils;

import java.text.DateFormat;
import java.util.Date;
import java.util.logging.*;
import java.util.jar.*;
import javax.swing.*;
import java.io.*;
import java.net.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.*;
import org.xml.sax.SAXException;


/**
 * This class performs several actions useful for many parts of the application.
 * @author Isakiev Ruslan
 */
public class Tools {
	
	private Logger myLogger;
	private String myMainJarName;
	private static Tools myInstance;
	
	private Tools() {
		myLogger = Logger.getLogger("org.isakiev.fileManager");
		myMainJarName = "";
	}
	
	/**
	 * Gets tools' instance
	 * @return tools' instance
	 */
	public static Tools getTools() {
		if (myInstance == null) {
			myInstance = new Tools();
		}
		return myInstance;
	}
	
	/**
	 * Sets program's jar file name. (It is used for loading xml documents)
	 * @param mainJarName program's jar file name
	 */
	public void setMainJarName(String mainJarName) {
		myMainJarName = mainJarName;
	}
	
	/**
	 * Loads resource icon.
	 * @param path icon's path
	 * @return icon
	 */
	public ImageIcon loadIcon(String path) {
        URL imageURL = Tools.class.getResource(path); 
        if (imageURL != null) {
            return new ImageIcon(imageURL);
        } else {
        	myLogger.warning("Resource wasn't found! ("+path+")");
        	return new ImageIcon();
        }
    }
	
	/**
	 * Returns string representation of date according to default locale
	 * @param dateValue date & time
	 * @return string representation
	 */
	public static String getDateTimeString(long dateValue) {
		Date date = new Date(dateValue); 
		return DateFormat.getDateInstance(DateFormat.SHORT).format(date)+ "   " +
			   DateFormat.getTimeInstance(DateFormat.SHORT).format(date);
	}
	
	/**
	 * Returns string representation of time according to default locale
	 * @param dateValue date & time
	 * @return string representation
	 */
	public static String getTimeString(long dateValue) {
		Date date = new Date(dateValue); 
		return DateFormat.getTimeInstance(DateFormat.MEDIUM).format(date);
	}	
	
	/**
	 * Returns string representation of file size which fits it best
	 * @param fileSize file size
	 * @return string representation
	 */
	public static String getFileSizeString(long fileSize) {
		
		if (fileSize < 1024) {
			return String.format("%1$d",fileSize) + " bytes";
		} else if (fileSize < 1024*1024) {
			return String.format("%1$1.2f",new Double(fileSize)/1024) + " Kb" ;
		} else if (fileSize < 1024*1024*1024) {
			return String.format("%1$1.2f",new Double(fileSize)/(1024*1024)) + " Mb" ;
		} else {
			return String.format("%1$1.2f",new Double(fileSize)/(1024*1024*1024)) + " Gb" ;
		}
	}
	
	/**
	 * Loads xml document from the stream. 
	 * @param stream stream to read from
	 * @return xml document or <code>null</code> if exception occured.
	 */
	private Document loadXMLDocumentFromStream(InputStream stream) {
		
		if (stream == null) {
			myLogger.warning("Input stream is null. 'Null' document is returned.");
			return null;
        }

		Document document;
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setNamespaceAware(true);
			DocumentBuilder builder = factory.newDocumentBuilder();

			document = builder.parse(stream);

		} catch (SAXException e) {
			myLogger.log(Level.SEVERE, "Cannot parse document! " + e.getMessage(), e);
			return null;
		} catch (IOException e) {
			myLogger.log(Level.SEVERE, "IOException! " + e.getMessage(), e);
			return null;
		} catch (ParserConfigurationException e) {
			myLogger.log(Level.SEVERE, "Cannot create parser! " + e.getMessage(), e);
			return null;
		}
		return document;
	}
	
	/**
	 * Loads xml document from file. 
	 * @param fileName xml document file name 
	 * @return xml document or <code>null</code> if exception occured or file not found.
	 */
	public Document loadXMLDocumentFromFile(String fileName) {
		
		if (myLogger.isLoggable(Level.FINE)) {
			myLogger.finer("Loading xml document from file \"" + fileName +"\"");
		}
		
		InputStream stream = null;
		try {
			stream = new FileInputStream(new File(fileName)); 
			return loadXMLDocumentFromStream(stream);
		} catch (FileNotFoundException fnfe) {
			myLogger.log(Level.WARNING,"File \"" + fileName +"\" wasn't found!",fnfe);
		} finally {
			if (stream != null) {
				try {
					stream.close();
				} catch (IOException e) {}
			}
		}
		return null;
	}
	
	/**
	 * Loads xml document from JAR-archive. 
	 * @param jarName archive, containing xml document
	 * @param fileName xml document file name 
	 * @return xml document or <code>null</code> if exception occured.
	 */
	private Document loadXMLDocumentFromJar(String jarName, String fileName) {
		
		if (myLogger.isLoggable(Level.FINE)) {
			myLogger.finer("Loading xml document from file \"" + fileName + "\" in JAR-archive \"" + jarName + "\"");
		}

		InputStream stream = null;
		try {
			JarFile jar = new JarFile(jarName);
			JarEntry entry = jar.getJarEntry(fileName);
			if (entry == null) {
				myLogger.warning("Entry \"" + fileName +"\" wasn't found! Null document was returned.");
				return null;
			} else {
				stream = jar.getInputStream(entry);
				return loadXMLDocumentFromStream(stream);
			}
		} catch (IOException ioe) {
			myLogger.log(Level.WARNING,"IOException! " + ioe.getMessage(), ioe);
		} finally {
			if (stream != null) {
				try {
					stream.close();
				} catch (IOException e) {}
			}
		}
		return null;
	}
	
	/**
	 * Tries to load xml document from file in program's jar archive,
	 * if program is not in jar archive method loads xnl from file. 
	 * @param mainJarName program's jar archive
	 * @param fileName xml document file name 
	 * @return xml document or <code>null</code> if exception occured.
	 */
	public Document loadXMLDocument(String fileName) {
		
		//Document doc = loadXMLDocumentFromJar(myMainJarName, fileName);
		//if (doc == null) {
		//	doc = loadXMLDocumentFromFile(fileName);
		//}
		
		URI uri = null;
		try {
			uri = Tools.class.getResource("/"+fileName).toURI();
		} catch (URISyntaxException e) {}
		if (uri != null) {
			String uriString = uri.toString();
			//System.out.println("URI => "+uriString);
			int p1 = uriString.indexOf("jar:file:");
			int p2 = uriString.lastIndexOf("!");
			if (p1 == -1) {
				return loadXMLDocumentFromFile(fileName);
			} else {
				String jarName = uriString.substring(p1+9, p2);
				//System.out.println(jarName);
				return loadXMLDocumentFromJar(jarName, fileName);
			}
		} 
		return null;
	}
	
	/**
	 * Saves xml document to file.
	 * @param document xml document
	 * @param fileName file name
	 * @return <code>true</code> if saved successfully
	 */
	public boolean saveXMLDocument(Document document, String fileName) {
		Logger logger = Logger.getLogger("org.isakiev.fileManager");
		try {
			Transformer transformer = TransformerFactory.newInstance().newTransformer();
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			//System.out.println(document.getDocumentElement().getNodeName());
			transformer.transform(new DOMSource(document), new StreamResult(new File(fileName)));
		} catch (TransformerConfigurationException tce) {
			logger.log(Level.SEVERE, "Cannot create transformer \"" + fileName + "\"", tce);
			return false;
		} catch (TransformerException te) {
			logger.log(Level.SEVERE, "Cannot transform document \"" + fileName + "\"", te);
			return false;
		} 
		return true;
	}
	
}
